#coding=utf8

'''
CloudControl CLI RPC Handler
'''

from cccli.exception import *
from cccli.printer import Printer, color
from sjrpc.utils import RpcHandler
import cccli
import os, os.path
import subprocess
import platform


class CliHandler(RpcHandler):
    '''Handle RPC incoming request'''

    def quit(self, rpc=None):
        '''Call back function on server disconnect'''
        Printer().warn("Disconnected from server!")

    def wall(self, sender, message):
        '''Receive a wall from the server'''
        Printer().out("\n\nWall from: %s\n\n%s\n\n" % (sender, message))

    def get_tags(self, tags):
        '''Return a dict of local tags'''
        ans = {}
        for tag in tags:
            method = "_get_tag_%s" % tag
            if hasattr(self, method):
                ans[tag] = getattr(self, method)()
        return ans

    def _get_tag_version(self):
        '''Return tag version'''
        return cccli.version

    def _get_tag_os(self):
        '''Return Operating system tag'''
        return platform.system().lower()

    def _get_tag_uname(self):
        '''Return uname output'''
        try:
            p = subprocess.Popen(["uname", "-a"], close_fds=True, shell=False, stdout=subprocess.PIPE)
            return  p.stdout.read().rstrip("\n")
        except Exception:
            return ""

    def _get_tag_uptime(self):
        '''Return uptime output'''
        try:
            return int(float(open("/proc/uptime", "r").readline().split()[0]))
        except Exception:
            return ""
